/*
 * Implementation of Hungarian method (Harold Kuhn, 1955)
 * Copyright (C) 2009 Zhi Yan <yz@ai.univ-paris8.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <iostream>
#include "hungarian.hpp"

int main(int argc, char *argv[]) {
  static const int N = 4;
  
  int data[N][N] = {
    {90, 75, 75, 80},
    {35, 85, 55, 65},
    {125, 95, 90, 105},
    {45, 110, 95, 115}
  };
  
  double **matrix;
  int i, j;
  
  std::list<Element> assignment;
  std::list<Element>::iterator it;
  
  // construct problem matrix
  matrix = new double*[N];
  for(i = 0; i < N; i++)
    matrix[i] = new double[N];
  
  for(i = 0; i < N; i++)
    for(j = 0; j < N; j++)
      matrix[i][j] = (double)data[i][j];
  
  // hungarian method process
  hungarianStep1(matrix, N);
  while(!hungarianStep2(matrix, N, assignment))
    hungarianStep3(matrix, N);
  
  // print assignment
  std::cout << "\n--- assignment ---\n";
  for(it = assignment.begin(); it != assignment.end(); it++)
    std::cout << "row " << it->getRow() << " <---> " << "column " << it->getColumn() << "\n";
  std::endl(std::cout);
  
  // release storage space for problem matrix
  for(i = 0; i < N; i++)
    delete [] matrix[i];
  delete [] matrix;
  matrix = NULL;
  
  return 0;
}
